package enseirb.myinpulse;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import enseirb.myinpulse.model.Administrator;
import enseirb.myinpulse.model.Project;
import enseirb.myinpulse.service.AdminApiService;
import enseirb.myinpulse.service.database.AdministratorService;
import enseirb.myinpulse.service.database.ProjectService;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

@SpringBootTest
@Transactional
public class AdminApiServiceTest {
    @Autowired private AdminApiService adminApiService;

    @BeforeAll
    static void setup(
            @Autowired AdministratorService administratorService,
            @Autowired ProjectService projectService) {
        administratorService.addAdministrator(
                new Administrator(
                        "admin",
                        "admin",
                        "testAdminEmpty@example.com",
                        "testAdmin@example.com",
                        ""));
        administratorService.addAdministrator(
                new Administrator(
                        "admin2",
                        "admin2",
                        "testAdminFull@example.com",
                        "testAdmin@example.com",
                        ""));
        projectService.addNewProject(
                new Project(
                        "sampleProjectAdminApiService",
                        null,
                        LocalDate.now(),
                        "ONGOING",
                        administratorService.getAdministratorByPrimaryMain(
                                "testAdminFull@example.com")));
    }

    @Test
    void getProjectOfAdminIsEmpty() {
        Iterable<Project> projects =
                adminApiService.getProjectsOfAdmin("testAdminEmpty@example.com");
        List<Project> l = new ArrayList<>();
        projects.forEach(l::add);
        assertEquals(0, l.size());
    }

    @Test
    void getProjectOfInexistantAdminFails() {
        String nonExistentAdminEmail = "testInexistantAdmin@example.com";

        assertThrows(
                ResponseStatusException.class,
                () -> {
                    adminApiService.getProjectsOfAdmin(nonExistentAdminEmail);
                });
    }

    @Test
    void getProjectOfAdminNotEmpty() {
        Iterable<Project> projects =
                adminApiService.getProjectsOfAdmin("testAdminFull@example.com");
        List<Project> l = new ArrayList<>();
        projects.forEach(l::add);
        assertEquals(1, l.size());
        Project p = l.getFirst();
        assertEquals(p.getProjectName(), "sampleProjectAdminApiService");
    }
}
